@@ -62,12 +62,12 @@ Notebook_Data::~Notebook_Data()
   remove_view(&m_Box_Details);
 }
 
-bool Notebook_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+bool Notebook_Data::init_db_details(const Glib::ustring& table_name, const Glib::ustring& where_clause)
 {
-  m_table_name = strTableName;
+  m_table_name = table_name;
 
   bool result = false;
-  //strWhereClause is only used as a result of a find.
+  //where_clause is only used as a result of a find.
 
   //Performance optimisation:
   //Keep the connection open during all these operations:
@@ -76,9 +76,9 @@ bool Notebook_Data::init_db_details(const Glib::ustring& strTableName, const Gli
 
     const Glib::ustring old_where_clause = m_Box_List.get_where_clause();
     //std::cout << "  old_where_clause=" << old_where_clause << std::endl;
-    //std::cout << "  strWhereClause=" << strWhereClause << std::endl;
-    const bool new_find_set = (strWhereClause != old_where_clause);
-    result = m_Box_List.init_db_details(m_table_name, strWhereClause);
+    //std::cout << "  where_clause=" << where_clause << std::endl;
+    const bool new_find_set = (where_clause != old_where_clause);
+    result = m_Box_List.init_db_details(m_table_name, where_clause);
     //m_Box_List.load_from_document();
 
     //Show the previously-shown record, if there is one, if this is not a new found-set (via a new where_clause)
@@ -90,15 +90,25 @@ bool Notebook_Data::init_db_details(const Glib::ustring& strTableName, const Gli
 
       if(!new_find_set)
       {
+        //std::cout << "debug: no new_found_set" << std::endl;
         primary_key_for_details = document->get_layout_record_viewed(m_table_name, m_Box_Details.get_layout_name());
       }
+      else
+      {
+         //std::cout << "debug: new_found_set" << std::endl;
+      }
 
       if(GlomConversions::value_is_empty(primary_key_for_details))
       {
         //Make sure that the details view is not empty, if there are any records to show:
         primary_key_for_details = m_Box_List.get_primary_key_value_selected();
+        //std::cout << "debug:  m_Box_List.get_primary_key_value_selected()=" << primary_key_for_details.to_string() << std::endl;
         if(GlomConversions::value_is_empty(primary_key_for_details))
+        {
+          //std::cout << "debug: calling list.get_primary_key_value_first()" << std::endl;
           primary_key_for_details = m_Box_List.get_primary_key_value_first();
+          //std::cout << "  debug:  result=" <<  primary_key_for_details.to_string() << std::endl;
+        }
       }
 
       m_Box_Details.init_db_details(m_table_name, primary_key_for_details);
@@ -151,9 +161,9 @@ void Notebook_Data::on_list_user_requested_details(const Gnome::Gda::Value& prim
   set_current_page(m_iPage_Details);
 }
 
-void Notebook_Data::on_Details_user_requested_related_details(const Glib::ustring& strTableName, Gnome::Gda::Value primary_key_value)
+void Notebook_Data::on_Details_user_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
 {
-  signal_record_details_requested().emit(strTableName, primary_key_value);
+  signal_record_details_requested().emit(table_name, primary_key_value);
 
   /*
   //Show a different table:
@@ -164,6 +174,14 @@ void Notebook_Data::on_Details_user_requested_related_details(const Glib::ustrin
   */
 }
 
+void Notebook_Data::set_current_view(dataview view)
+{
+  if(view == DATA_VIEW_List)
+    set_current_page(m_iPage_List);
+  else
+    set_current_page(m_iPage_Details);
+}
+
 void Notebook_Data::select_page_for_find_results()
 {
   if(m_Box_List.get_showing_multiple_records())
@@ -213,7 +231,7 @@ Notebook_Data::dataview Notebook_Data::get_current_view() const
   const int current_page = get_current_page();
 
   dataview result = DATA_VIEW_Details;
-  if(current_page == 1)
+  if(current_page == (int)m_iPage_List)
     result = DATA_VIEW_List;
 
   return result;
