@@ -46,6 +46,8 @@ void Box_DB_Table_Definition::init()
 
   pack_start(m_AddDel);
   m_colName = m_AddDel.add_column(_("Name"));
+  m_AddDel.prevent_duplicates(m_colName); //Don't allow adding of fields that already exist.
+  m_AddDel.set_prevent_duplicates_warning(_("This field already exists. Please choose a different field name"));
 
   m_colTitle = m_AddDel.add_column(_("Title"));
 
@@ -58,8 +60,8 @@ void Box_DB_Table_Definition::init()
   AddDel::type_vecStrings vecTypes;
   for(Field::type_map_type_names ::iterator iter = mapFieldTypes.begin(); iter != mapFieldTypes.end();++iter)
   {
-    const Glib::ustring& strName = (*iter).second;
-    vecTypes.push_back(strName);
+    const Glib::ustring& name = (*iter).second;
+    vecTypes.push_back(name);
   }
 
   m_AddDel.set_column_choices(m_colType, vecTypes);
@@ -151,10 +153,10 @@ bool Box_DB_Table_Definition::fill_from_database()
 
 void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
 {
-  Glib::ustring strName = m_AddDel.get_value(row, m_colName);
-  if(!strName.empty())
+  Glib::ustring name = m_AddDel.get_value(row, m_colName);
+  if(!name.empty())
   {
-    bool bTest = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" ADD \"" + strName + "\" NUMERIC" ); //TODO: Get schema type for Field::TYPE_NUMERIC
+    bool bTest = Query_execute( "ALTER TABLE \"" + m_table_name + "\" ADD \"" + name + "\" NUMERIC" ); //TODO: Get schema type for Field::TYPE_NUMERIC
     if(bTest)
     {
       //Show the new field (fill in the other cells):
@@ -165,8 +167,8 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
 
       //This must match the SQL statement above:
       sharedptr<Field> field(new Field());
-      field->set_name(strName);
-      field->set_title( util_title_from_string(strName) ); //Start with a title that might be useful.
+      field->set_name(name);
+      field->set_title( util_title_from_string(name) ); //Start with a title that might be useful.
       field->set_glom_type(Field::TYPE_NUMERIC);
 
       Gnome::Gda::FieldAttributes field_info = field->get_field_info();
@@ -191,13 +193,13 @@ void Box_DB_Table_Definition::on_adddel_delete(const Gtk::TreeModel::iterator& r
 
   for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfterEnd; ++iter)
   {
-    Glib::ustring strName = m_AddDel.get_value_key(iter);
-    if(!strName.empty())
+    Glib::ustring name = m_AddDel.get_value_key(iter);
+    if(!name.empty())
     {
-      Query_execute( "ALTER TABLE \"" + m_strTableName + "\" DROP COLUMN \"" + strName + "\"");
+      Query_execute( "ALTER TABLE \"" + m_table_name + "\" DROP COLUMN \"" + name + "\"");
 
       //Remove it from all layouts, reports, etc:
-      get_document()->remove_field(m_strTableName, strName);
+      get_document()->remove_field(m_table_name, name);
     }
   }
 
@@ -214,7 +216,7 @@ void Box_DB_Table_Definition::on_adddel_changed(const Gtk::TreeModel::iterator&
     //Glom-specific stuff: //TODO_portiter
     const Glib::ustring strFieldNameBeingEdited = m_AddDel.get_value_key(row);
 
-    sharedptr<const Field> constfield = pDoc->get_field(m_strTableName, strFieldNameBeingEdited);
+    sharedptr<const Field> constfield = pDoc->get_field(m_table_name, strFieldNameBeingEdited);
     m_Field_BeingEdited = constfield;
 
     //Get DB field info: (TODO: This might be unnecessary).
@@ -251,7 +253,7 @@ void Box_DB_Table_Definition::on_adddel_edit(const Gtk::TreeModel::iterator& row
   sharedptr<const Field> constfield = get_field_definition(row);
   m_Field_BeingEdited = constfield;
 
-  m_pDialog->set_field(m_Field_BeingEdited, m_strTableName);
+  m_pDialog->set_field(m_Field_BeingEdited, m_table_name);
 
   //m_pDialog->set_modified(false); //Disable [Apply] at start.
 
@@ -270,7 +272,7 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
   Document_Glom* pDoc = static_cast<Document_Glom*>(get_document());
   if(pDoc)
   {
-    Document_Glom::type_vecFields vecFields= pDoc->get_table_fields(m_strTableName);
+    Document_Glom::type_vecFields vecFields= pDoc->get_table_fields(m_table_name);
     Document_Glom::type_vecFields::iterator iterFind = std::find_if( vecFields.begin(), vecFields.end(), predicate_FieldHasName<Field>(strFieldNameBeforeEdit) );
 
     if((iterFind != vecFields.end()) && (*iterFind)) //If it was found:
@@ -289,14 +291,14 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
 
   //Start with original definitions, so that we preserve things like UNSIGNED.
   //TODO maybe use document's fieldinfo instead of m_vecFields.
-  sharedptr<const Field> field_temp = get_fields_for_table_one_field(m_strTableName, strFieldNameBeforeEdit);
+  sharedptr<const Field> field_temp = get_fields_for_table_one_field(m_table_name, strFieldNameBeforeEdit);
   if(field_temp)
   {
     Gnome::Gda::FieldAttributes fieldInfo = field_temp->get_field_info();
 
     //Name:
-    const Glib::ustring strName = m_AddDel.get_value(row, m_colName);
-    fieldInfo.set_name(strName);
+    const Glib::ustring name = m_AddDel.get_value(row, m_colName);
+    fieldInfo.set_name(name);
 
     //Title:
     const Glib::ustring title = m_AddDel.get_value(row, m_colTitle);
@@ -342,8 +344,27 @@ void Box_DB_Table_Definition::on_Properties_apply()
     }
     else
     {
-      change_definition(m_Field_BeingEdited, field_New);
-      m_Field_BeingEdited = field_New;
+      //Warn about recalculation:
+      bool change_def = true;
+      if(field_New->get_has_calculation())
+      {
+        if(field_New->get_calculation() != m_Field_BeingEdited->get_calculation())
+        {
+          //TODO: Only show this when there are > 100 records?
+          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Recalculation Required")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE);
+          dialog.set_secondary_text(_("You have changed the calculation used by this field so Glom must recalculate the value in all records. If the table contains many records then this could take a long time."));
+          //dialog.set_transient_for(*get_app_window());
+          dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+          dialog.add_button(_("Recalculate"), Gtk::RESPONSE_OK);
+          change_def = (dialog.run() == Gtk::RESPONSE_OK);
+        }
+      }
+
+      if(change_def)
+      {
+        change_definition(m_Field_BeingEdited, field_New);
+        m_Field_BeingEdited = field_New;
+      }
     }
 
     //Update the list:
@@ -371,7 +392,7 @@ void Box_DB_Table_Definition::change_definition(const sharedptr<const Field>& fi
   if(pDoc)
   {
     //Get Table's fields:
-    Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_strTableName);
+    Document_Glom::type_vecFields vecFields = pDoc->get_table_fields(m_table_name);
 
     //Find old field:
     const Glib::ustring field_name_old = fieldOld->get_name();
@@ -388,12 +409,20 @@ void Box_DB_Table_Definition::change_definition(const sharedptr<const Field>& fi
       vecFields.push_back( glom_sharedptr_clone(field) );
     }
 
-    pDoc->set_table_fields(m_strTableName, vecFields);
+    pDoc->set_table_fields(m_table_name, vecFields);
 
     //Update field names where they are used in relationships or on layouts:
     if(field_name_old != field->get_name())
     {
-      pDoc->change_field_name(m_strTableName, field_name_old, field->get_name());
+      pDoc->change_field_name(m_table_name, field_name_old, field->get_name());
+    }
+
+    //Recalculate if necessary:
+    if(field->get_has_calculation())
+    {
+      const Glib::ustring calculation = field->get_calculation();
+      if(calculation != fieldOld->get_calculation())
+        calculate_field_in_all_records(m_table_name, field);
     }
   }
 
@@ -409,20 +438,7 @@ void Box_DB_Table_Definition::change_definition(const sharedptr<const Field>& fi
 
 void Box_DB_Table_Definition::fill_fields()
 {
-  m_vecFields = get_fields_for_table(m_strTableName);
-}
-
-void  Box_DB_Table_Definition::postgres_add_column(const sharedptr<const Field>& field, bool not_extras)
-{
-  const bool bTest = Query_execute(  "ALTER TABLE \"" + m_strTableName + "\" ADD \"" + field->get_name() + "\" " +  field->get_sql_type() );
-  if(bTest)
-  {
-    if(not_extras)
-    {
-      //We must do this separately:
-      postgres_change_column_extras(field, field, true /* set them even though the fields are the same */);
-    }
-  }
+  m_vecFields = get_fields_for_table(m_table_name);
 }
 
 void  Box_DB_Table_Definition::postgres_change_column(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
@@ -438,10 +454,11 @@ void  Box_DB_Table_Definition::postgres_change_column(const sharedptr<const Fiel
   else
   {
     //Change other stuff, without changing the type:
-    postgres_change_column_extras(field_old, field);
+    postgres_change_column_extras(m_table_name, field_old, field);
   }
 }
 
+
 void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
 {
   sharedptr<SharedConnection> sharedconnection = connect_to_server();
@@ -463,7 +480,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
       //    COMMIT;
 
       Glib::RefPtr<Gnome::Gda::Transaction> transaction = Gnome::Gda::Transaction::create("glom_transaction_change_field_type");
-      bool test = gda_connection->begin_transaction(transaction);
+      const bool test = gda_connection->begin_transaction(transaction);
       if(test)
       {
         //TODO: Warn about a delay, and possible loss of precision, before actually doing this.
@@ -471,7 +488,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
 
         sharedptr<Field> fieldTemp = glom_sharedptr_clone(field);
         fieldTemp->set_name("glom_temp_column");
-        postgres_add_column(fieldTemp); //This might also involves several commands.
+        postgres_add_column(m_table_name, fieldTemp); //This might also involves several commands.
 
 
         bool conversion_failed = false;
@@ -486,49 +503,89 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
           WHERE _aaa <> '     ';  
           */
           Glib::ustring conversion_command;
+          const Glib::ustring field_name_old_quoted = "\"" + field_old->get_name() + "\"";
+          const Field::glom_field_type old_field_type = field_old->get_glom_type();
           switch(field->get_glom_type())
           {
-            case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is numeric.
+            case Field::TYPE_BOOLEAN: //CAST does not work if the destination type is boolean.
             {
-              conversion_command = "FALSE"; //TODO: Find a way to convert: "to_number( " + field_old->get_name() + ", '999999999.99' )";
+              if(old_field_type == Field::TYPE_NUMERIC)
+              {
+                conversion_command = "(CASE WHEN " + field_name_old_quoted + " >0 THEN true "
+                                           "WHEN " + field_name_old_quoted + " = 0 THEN false "
+                                           "WHEN " + field_name_old_quoted + " IS NULL THEN false END)";
+              }
+              else if(old_field_type == Field::TYPE_TEXT)
+                conversion_command = "(" + field_name_old_quoted + " !~~* \'false\')"; // !~~* means ! ILIKE.
+              else //Dates and Times:
+                conversion_command = "(" + field_name_old_quoted + " IS NOT NULL')";
+
               break;
             }
             case Field::TYPE_NUMERIC: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_number( \"" + field_old->get_name() + "\", '999999999.99' )";
+              if(old_field_type == Field::TYPE_BOOLEAN)
+              {
+                conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN 1 "
+                                           "WHEN " + field_name_old_quoted + " = false THEN 0 "
+                                           "WHEN " + field_name_old_quoted + " IS NULL THEN 0 END)";
+              }
+              else
+              {
+                //We use to_number, with textcat() so that to_number always has usable data.
+                //Otherwise, it says 
+                //invalid input syntax for type numeric: " "
+                //
+                //We must use single quotes with the 0, otherwise it says "column 0 does not exist.".
+                conversion_command = "to_number( textcat(\'0\', " + field_name_old_quoted + "), '999999999.99999999' )";
+              }
+
               break;
             }
             case Field::TYPE_DATE: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_date( \"" + field_old->get_name() + "\", 'YYYYMMDD' )"; //TODO: standardise date storage format.
+              conversion_command = "to_date( " + field_name_old_quoted + ", 'YYYYMMDD' )"; //TODO: standardise date storage format.
               break;
             }
             case Field::TYPE_TIME: //CAST does not work if the destination type is numeric.
             {
-              conversion_command = "to_timestamp( \"" + field_old->get_name() + "\", 'HHMMSS' )";  //TODO: standardise time storage format.
+              conversion_command = "to_timestamp( " + field_name_old_quoted + ", 'HHMMSS' )";  //TODO: standardise time storage format.
               break;
             }
             default:
             {
-              conversion_command = "CAST(\"" +  field_old->get_name() + "\" AS " + field->get_sql_type() + ")";
+              //To Text:
+
+              //bool to text:
+              if(old_field_type == Field::TYPE_BOOLEAN)
+              {
+                 conversion_command = "(CASE WHEN " + field_name_old_quoted + " = true THEN \'true\' "
+                                            "WHEN " + field_name_old_quoted + " = false THEN \'false\' "
+                                            "WHEN " + field_name_old_quoted + " IS NULL THEN \'false\' END)";
+              }
+              else
+              {
+                //This works for most to-text conversions:
+                conversion_command = "CAST(\"" +  field_old->get_name() + "\" AS " + field->get_sql_type() + ")";
+              }
               break;
             }
           }
 
-          Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "UPDATE \"" + m_strTableName + "\" SET \"" + fieldTemp->get_name() + "\" = " + conversion_command );  //TODO: Not full type details.
+          Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "UPDATE \"" + m_table_name + "\" SET \"" + fieldTemp->get_name() + "\" = " + conversion_command );  //TODO: Not full type details.
           if(!datamodel)
             conversion_failed = true;
         }
 
         if(!conversion_failed)
         {
-          Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" DROP COLUMN \"" +  field_old->get_name() + "\"");
+          Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "ALTER TABLE \"" + m_table_name + "\" DROP COLUMN \"" +  field_old->get_name() + "\"");
           if(datamodel)
           {
-            datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" RENAME COLUMN \"" + fieldTemp->get_name() + "\" TO \"" + field->get_name() + "\"");
+            datamodel = Query_execute( "ALTER TABLE \"" + m_table_name + "\" RENAME COLUMN \"" + fieldTemp->get_name() + "\" TO \"" + field->get_name() + "\"");
             if(datamodel)
             {
-              bool test = gda_connection->commit_transaction(transaction);
+              const bool test = gda_connection->commit_transaction(transaction);
               if(!test)
               {
                 handle_error();
@@ -542,88 +599,10 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
     }  /// If the datatype has changed:
 
     if(!new_column_created) //We don't need to change anything else if everything was alrady correctly set as a new column:
-      postgres_change_column_extras(field_old, field);
+      postgres_change_column_extras(m_table_name, field_old, field);
   }
 }
 
 
-void Box_DB_Table_Definition::postgres_change_column_extras(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field, bool set_anyway)
-{
-  //Gnome::Gda::FieldAttributes field_info = field->get_field_info();
-  //Gnome::Gda::FieldAttributes field_info_old = field_old->get_field_info();
-
-  if(field->get_name() != field_old->get_name())
-  {
-     Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" RENAME COLUMN \"" + field_old->get_name() + "\" TO \"" + field->get_name() + "\"" );
-     if(!datamodel)
-     {
-       handle_error();
-       return;
-     }
-  }
-
-  if(set_anyway || (field->get_primary_key() != field_old->get_primary_key()))
-  {
-    //TODO: Check that there is only one primary key.
-    Glib::ustring add_or_drop = "ADD";
-    if(field_old->get_primary_key() == false)
-      add_or_drop = "DROP";
-
-    Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" " + add_or_drop + " PRIMARY KEY (\"" + field->get_name() + "\")");
-    if(!datamodel)
-    {
-      handle_error();
-      return;
-    }
-  }
-
-  if( !field->get_primary_key() ) //Postgres automatically makes primary keys unique, so we do not need to do that separately if we have already made it a primary key
-  {
-    if(set_anyway || (field->get_unique_key() != field_old->get_unique_key()))
-    {
-       /* TODO: Is there an easier way than adding an index manually?
-       Glib::RefPtr<Gnome::Gda::DataModel>  datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" RENAME COLUMN " + field_info_old.get_name() + " TO " + field_info.get_name() );
-       if(!datamodel)
-       {
-         handle_error();
-         return;
-       }
-       */
-    }
-
-    Gnome::Gda::Value default_value = field->get_default_value();
-    Gnome::Gda::Value default_value_old = field_old->get_default_value();
-
-    if(!field->get_auto_increment()) //Postgres auto-increment fields have special code as their default values.
-    {
-      if(set_anyway || (default_value != default_value_old))
-      {
-        Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute( "ALTER TABLE \"" + m_strTableName + "\" ALTER COLUMN \""+ field->get_name() + "\" SET DEFAULT " + field->sql(field->get_default_value()) );
-        if(!datamodel)
-        {
-          handle_error();
-          return;
-        }
-      }
-    }
-  }
-
-  /* This should have been dealt with by postgres_change_column_type(), because postgres uses a different ("serial") field type for auto-incrementing fields.
-  if(field_info.get_auto_increment() != field_info_old.get_auto_increment())
-  {
-
-  }
-  */
- 
-   /*
-    //If the not-nullness has changed:
-    if( set_anyway ||  (field->get_field_info().get_allow_null() != field_old->get_field_info().get_allow_null()) )
-    {
-      Glib::ustring nullness = (field->get_field_info().get_allow_null() ? "NULL" : "NOT NULL");
-      Query_execute(  "ALTER TABLE " + m_strTableName + " ALTER COLUMN " + field->get_name() + "  SET " + nullness);
-    }
-  */ 
-}
-
 
 
