@@ -23,12 +23,13 @@
 
 #include "../../connectionpool.h"
 #include "../../data_structure/glomconversions.h" //For util_build_sql
+#include "../../utils.h"
 
 
 DbTreeModelRow::DbTreeModelRow()
-: m_placeholder(false),
-  m_values_retrieved(false),
-  m_removed(false)
+: m_values_retrieved(false),
+  m_removed(false),
+  m_extra(false)
   //m_data_model_row_number(false)
 {
 
@@ -48,12 +49,23 @@ void DbTreeModelRow::fill_values_if_necessary(DbTreeModel& model, int row)
       }
 
       m_key = model.m_gda_datamodel->get_value_at(model.m_column_index_key, row);
+
+      m_extra = false;
+      m_removed = false;
     }
     else
     {
       //g_warning("DbTreeModelRow::fill_values_if_necessary(): Non-db row.");
-      //It is an extra row, added with append().
-      //Use the default values.
+      if(m_extra)
+      {
+        //It is an extra row, added with append().
+        //Use the default values.
+      }
+      else if(!m_removed)
+      {
+        //It must be the last blank placeholder row.
+        //m_placeholder = true;
+      }
     }
 
   }
@@ -140,6 +152,7 @@ DbTreeModel::DbTreeModel(const Gtk::TreeModelColumnRecord& columns, const Glib::
   m_data_model_columns_count(0),
   m_count_extra_rows(0),
   m_count_removed_rows(0),
+  m_get_records(get_records),
   m_stamp(1), //When the model's stamp != the iterator's stamp then that iterator is invalid and should be ignored. Also, 0=invalid
   m_pGlueList(0)
 {
@@ -157,31 +170,7 @@ DbTreeModel::DbTreeModel(const Gtk::TreeModelColumnRecord& columns, const Glib::
 
   g_assert(m_columns_count == column_fields.size());
 
-  //Connect to database:
-  ConnectionPool* connection_pool = ConnectionPool::get_instance();
-  if(connection_pool)
-  {
-     m_connection = connection_pool->connect();
-  }
-
-  if(m_connection && !table_name.empty() && get_records)
-  {
-    const Glib::ustring sql_query = util_build_sql_select_with_where_clause(table_name, column_fields, where_clause);
-
-    std::cout << "DbTreeModel: Executing SQL: " << sql_query << std::endl;
-    m_gda_datamodel = m_connection->get_gda_connection()->execute_single_command(sql_query);
-    if(!m_gda_datamodel)
-    {
-      //TODO: handle_error();
-    }
-    else
-    {
-      m_data_model_rows_count = m_gda_datamodel->get_n_rows(); //TODO_Performance: This probably gets all the data.
-      m_data_model_columns_count = m_gda_datamodel->get_n_columns();
-    }
-  }
-  else
-    m_data_model_columns_count = m_columns_count;
+  //refresh_from_database();
 }
 
 DbTreeModel::~DbTreeModel()
@@ -195,6 +184,69 @@ Glib::RefPtr<DbTreeModel> DbTreeModel::create(const Gtk::TreeModelColumnRecord&
   return Glib::RefPtr<DbTreeModel>( new DbTreeModel(columns, table_name, column_fields, column_index_key, get_records, where_clause) );
 }
 
+bool DbTreeModel::refresh_from_database(const Glib::ustring& where_clause)
+{
+  m_where_clause = where_clause;
+
+  if(!m_get_records)
+    return false;
+
+
+  clear(); //Clear existing shown records.
+
+  //Connect to database:
+  ConnectionPool* connection_pool = ConnectionPool::get_instance();
+  if(connection_pool)
+  {
+     m_connection = connection_pool->connect();
+  }
+
+  if(m_connection && !m_table_name.empty() && m_get_records)
+  {
+    const Glib::ustring sql_query = GlomUtils::build_sql_select_with_where_clause(m_table_name, m_column_fields, m_where_clause);
+
+    //std::cout << "DbTreeModel: Executing SQL: " << sql_query << std::endl << std::endl;
+    m_gda_datamodel = m_connection->get_gda_connection()->execute_single_command(sql_query);
+    if(!m_gda_datamodel)
+    {
+      m_data_model_rows_count = 0;
+      m_data_model_columns_count = m_columns_count;
+
+      std::cerr << "DbTreeModel::refresh_from_database(): Error while executing SQL" << std::endl <<
+                   "  " <<  sql_query << std::endl;
+
+      ConnectionPool::handle_error();
+      return false; //No records were found.
+    }
+    else
+    {
+      m_data_model_rows_count = m_gda_datamodel->get_n_rows(); //TODO_Performance: This probably gets all the data.
+      m_data_model_columns_count = m_gda_datamodel->get_n_columns();
+
+      /*
+      guint rows_to_get = 100;
+      if(rows_to_get > m_data_model_rows_count)
+        rows_to_get = m_data_model_rows_count;
+
+      for(guint i = 0; i < rows_to_get; ++i)
+      {
+        iterator iter;
+        const bool iter_is_valid = create_iterator(i, iter);
+        if(iter_is_valid)
+          row_inserted(get_path(iter), iter); //Allow the TreeView to respond to the addition.
+      }
+      */
+
+      return true;
+    }
+  }
+  else
+  {
+    m_data_model_columns_count = m_columns_count;
+    return false; //No records were found.
+  }
+}
+
 Gtk::TreeModelFlags DbTreeModel::get_flags_vfunc() const
 {
    return (Gtk::TreeModelFlags)(Gtk::TREE_MODEL_LIST_ONLY | Gtk::TREE_MODEL_ITERS_PERSIST);
@@ -228,8 +280,9 @@ void DbTreeModel::get_value_vfunc(const TreeModel::iterator& iter, int column, G
       //value_specific.init( Glib::Value< DbValue >::value_type() ); //TODO: Is there any way to avoid this step?
 
       type_datamodel_const_iter dataRowIter = get_datamodel_row_iter_from_tree_row_iter(iter);
-//g_warning("DbTreeModel::get_value_vfunc(): dataRowIter=%d", dataRowIter);
-      if(dataRowIter < (unsigned int)get_internal_rows_count()) //!= m_rows.end())
+      //g_warning("DbTreeModel::get_value_vfunc(): dataRowIter=%d, get_internal_rows_count=%d", dataRowIter, get_internal_rows_count());
+      const unsigned int internal_rows_count = get_internal_rows_count();
+      if( dataRowIter < internal_rows_count) //!= m_rows.end())
       {
         //const typeRow& dataRow = *dataRowIter;
 
@@ -239,23 +292,28 @@ void DbTreeModel::get_value_vfunc(const TreeModel::iterator& iter, int column, G
 
         DbTreeModelRow& row_details = m_map_rows[dataRowIter]; //Adds it if necessary.
         const int column_sql = column;
-        if(column_sql < (int)m_data_model_columns_count) //TODO_Performance: Remove the checks.
+        if(column_sql < (int)m_columns_count) //TODO_Performance: Remove the checks.
         {
-          if(true) //dataRowIter < (unsigned int)m_data_model_rows_count)
+          if( !(dataRowIter < (internal_rows_count - 1)))
+          {
+             //std::cout << "DbTreeModel::get_value_vfunc: row " << dataRowIter << " is placeholder" << std::endl;
+
+             //If it's after the database rows then it must be a placeholder row.
+             //We have only one of these because iter_n_root_children_vfunc() only adds 1 to the row count.
+             //row_details.m_placeholder = true;
+          }
+          else
           {
             result = row_details.get_value(const_cast<DbTreeModel&>(*this), column_sql, dataRowIter); //m_gda_datamodel->get_value_at(column_sql, dataRowIter); //dataRow.m_db_values[column];
           }
-          else
-            g_warning("DbTreeModel::get_value_vfunc: row out of bounds.");
         }
         else
-          g_warning("DbTreeModel::get_value_vfunc: column out of bounds: sql_col=%d, max=%d.", column_sql, m_data_model_columns_count);
+          g_warning("DbTreeModel::get_value_vfunc: column out of bounds: sql_col=%d, max=%d.", column_sql, m_columns_count);
 
 
         value_specific.set(result); //The compiler would complain if the type was wrong.
         value.init( Glib::Value< DbValue >::value_type() ); //TODO: Is there any way to avoid this step? Can't it copy the type as well as the value?
         value = value_specific;
-
       }
     }
   }
@@ -304,6 +362,7 @@ int DbTreeModel::iter_n_children_vfunc(const iterator& iter) const
 
 int DbTreeModel::iter_n_root_children_vfunc() const
 {
+  //std::cout << "iter_n_root_children_vfunc(): returning: " << get_internal_rows_count() - m_count_removed_rows + 1 << std::endl;
   return get_internal_rows_count() - m_count_removed_rows;
 }
 
@@ -327,7 +386,7 @@ bool DbTreeModel::iter_nth_child_vfunc(const iterator& parent, int /* n */, iter
 bool DbTreeModel::iter_nth_root_child_vfunc(int n, iterator& iter) const
 {
 //g_warning("DbTreeModel::iter_nth_root_child_vfunc(): n=%d", n);
- 
+
   //if(m_data_model_rows_count == 0)
   //  return false;  //There are no children.
 
@@ -351,7 +410,7 @@ bool DbTreeModel::iter_nth_root_child_vfunc(int n, iterator& iter) const
     //TODO_Performance:
     //Get the nth unremoved row:
     type_datamodel_iter row_iter = 0;
-    for(int child_n = 0; child_n <n; ++child_n)
+    for(int child_n = 0; child_n < n; ++child_n)
     {
       //Jump over hidden rows:
       while(m_map_rows[row_iter].m_removed)
@@ -424,9 +483,10 @@ bool DbTreeModel::create_iterator(const type_datamodel_iter& row_iter, DbTreeMod
 
   const guint count_all_rows = get_internal_rows_count();
 //g_warning("DbTreeModel::create_iterator(): row_iter=%d, count=%d", row_iter, count_all_rows);
-  if(row_iter >= count_all_rows) //row_iter == m_rows.end())
+  if(row_iter >= (count_all_rows)) //row_iter == m_rows.end()) //1 for the placeholder.
   {
-     //g_warning("DbTreeModel::create_iterator(): out of bounds: returning invalid iterator.");
+    //TreeView seems to use this to identify the last row.
+    //g_warning("DbTreeModel::create_iterator(): out of bounds: returning invalid iterator: row_iter=%d", row_iter);
     invalidate_iter(iter);
     return false;
   }
@@ -436,7 +496,7 @@ bool DbTreeModel::create_iterator(const type_datamodel_iter& row_iter, DbTreeMod
     //Store the std::list iterator in the GtkTreeIter:
     //See also iter_next_vfunc()
 
-    //g_warning("DbTreeModel::create_iterator(): Creating iter to row_index=%d", row_index);
+    //g_warning("DbTreeModel::create_iterator(): Creating iter to row_index=%d", row_iter);
     if(!m_pGlueList)
     {
      m_pGlueList = new GlueList();
@@ -547,7 +607,7 @@ void DbTreeModel::remember_glue_item(GlueItem* item) const
 
 int DbTreeModel::get_internal_rows_count() const
 {
-  return m_data_model_rows_count + m_count_extra_rows;
+  return m_data_model_rows_count + m_count_extra_rows + 1; //1 for placeholder.
 }
 
 DbTreeModel::iterator DbTreeModel::append()
@@ -580,7 +640,9 @@ DbTreeModel::iterator DbTreeModel::append()
   iterator iter;
   const bool iter_is_valid = create_iterator(row_iter, iter);
   if(iter_is_valid)
+  {
     row_inserted(get_path(iter), iter); //Allow the TreeView to respond to the addition.
+  }
 
   return iter;
 }
@@ -667,45 +729,46 @@ void DbTreeModel::set_key_value(const TreeModel::iterator& iter, const DbValue&
 }
 
 DbTreeModel::DbValue DbTreeModel::get_key_value(const TreeModel::iterator& iter) const
-{
-  //g_warning("DbTreeModel::g et_is_placeholder()");
+{   
   if(check_treeiter_validity(iter))
   {
     type_datamodel_iter dataRowIter = get_datamodel_row_iter_from_tree_row_iter(iter);
-    return m_map_rows[dataRowIter].m_key; //Adds it if necessary.
+    type_map_rows::iterator iterFind = m_map_rows.find(dataRowIter);
+    if(iterFind != m_map_rows.end())
+      return iterFind->second.m_key;
+    else
+    {
+      DbTreeModelRow& row_details = m_map_rows[dataRowIter]; //Adds it if necessary
+      row_details.fill_values_if_necessary(const_cast<DbTreeModel&>(*this), dataRowIter);
+      return row_details.m_key;
+    }
   }
 
   return DbValue();
 }
 
-
-
-void DbTreeModel::set_is_placeholder(const TreeModel::iterator& iter, bool val)
-{
-  //g_warning("DbTreeModel::set_is_placeholder(): val=%d", val);
-  if(check_treeiter_validity(iter))
-  {
-    type_datamodel_iter dataRowIter = get_datamodel_row_iter_from_tree_row_iter(iter);
-    m_map_rows[dataRowIter].m_placeholder = val;
-  }
-}
-
 bool DbTreeModel::get_is_placeholder(const TreeModel::iterator& iter) const
 {
   //g_warning("DbTreeModel::g et_is_placeholder()");
   if(check_treeiter_validity(iter))
   {
     type_datamodel_iter dataRowIter = get_datamodel_row_iter_from_tree_row_iter(iter);
-
-    type_map_rows::const_iterator iterFind = m_map_rows.find(dataRowIter);
-    if(iterFind != m_map_rows.end());
-      return iterFind->second.m_placeholder;
-      //return dataRowIter->m_placeholder;
+    return (dataRowIter == ((type_datamodel_iter)get_internal_rows_count() -1));
   }
 
   return false;
 }
 
+void DbTreeModel::set_is_not_placeholder(const TreeModel::iterator& iter)
+{
+  if(get_is_placeholder(iter))
+  {
+    //This row is an extra row (after a refresh it will probably be a row in the database).
+    //So now we need a new row to be the placeholder.
+    append();
+  }
+}
+
 void DbTreeModel::clear()
 {
   //m_rows.clear();
@@ -732,18 +795,19 @@ Gtk::TreeModel::iterator DbTreeModel::get_last_row()
   iterator result;
 
   //Find the last non-removed row:
-  const int rows_count = get_internal_rows_count();
+  int rows_count = get_internal_rows_count();
   if(rows_count)
   {
     type_datamodel_iter row = rows_count - 1;
+    --rows_count; //Ignore the placeholder.
 
-    //Step backwards until we find one that is not removed:
-    while(m_map_rows[row].m_removed)
+    //Step backwards until we find one that is not removed.
+    while((m_map_rows.find(row) != m_map_rows.end()) && m_map_rows[row].m_removed)
     {
       if(row)
         --row;
       else
-        return result; //failed, because there are no non-removed rows.
+        return result; //failed, because there are no non-removed row.
     }
 
     //g_warning("DbTreeModel::get_last_row(): returning row=%d", row);
@@ -752,3 +816,68 @@ Gtk::TreeModel::iterator DbTreeModel::get_last_row()
 
   return result;
 }
+
+Gtk::TreeModel::iterator DbTreeModel::get_placeholder_row()
+{
+  iterator result;
+
+  //Find the last non-removed row:
+  const int rows_count = get_internal_rows_count();
+  if(rows_count)
+  {
+    type_datamodel_iter row = rows_count - 1;
+
+    //Step backwards until we find one that is not removed:
+    while((m_map_rows.find(row) != m_map_rows.end()) && m_map_rows[row].m_removed)
+    {
+      if(row)
+        --row;
+      else
+      {
+        g_warning("bTreeModel::get_placeholder_row(): Placeholder row not found.");
+        return result; //failed, because there are no non-removed rows.
+      }
+    }
+
+    type_map_rows::const_iterator iter_map = m_map_rows.find(row);
+    if(iter_map != m_map_rows.end())
+    {
+      //g_warning("DbTreeModel::get_last_row(): returning row=%d", row);
+      create_iterator(row, result);
+    }
+  }
+
+  return result;
+}
+
+void DbTreeModel::get_record_counts(gulong& total, gulong& found) const
+{
+  if(m_gda_datamodel)
+  {
+    found = (gulong)m_gda_datamodel->get_n_rows();
+
+    if(m_where_clause.empty())
+      total = found;
+    else
+    {
+      //Ask the database how many records there are in the whole table:
+      //TODO: Apparently, this is very slow:
+      const Glib::ustring sql_query = "SELECT count(*) FROM \"" + m_table_name + "\"";
+      Glib::RefPtr<Gnome::Gda::DataModel> datamodel = m_connection->get_gda_connection()->execute_single_command(sql_query);
+
+      if(datamodel)
+      {
+        if(datamodel->get_n_rows())
+        {
+          Gnome::Gda::Value value = datamodel->get_value_at(0, 0);
+          total = (gulong)value.get_bigint(); //I discovered that it's a bigint by trying it.
+        }
+      }
+    }
+  }
+  else
+  {
+    total = 0;
+    found = 0;
+  }
+}
