@@ -21,17 +21,63 @@
 #include "utils.h"
 #include "connectionpool.h"
 #include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
+#include "data_structure/glomconversions.h"
+#include "frame_glom.h"
 #include <libxml++/libxml++.h>
 #include <libxslt/transform.h>
+//#include <libexslt/exslt.h> //For exsltRegisterAll().
 #include <libgnomevfsmm.h>
+#include <glibmm/i18n.h>
 #include <libgnome/gnome-url.h>
 #include <sstream> //For stringstream
 
+#include <iostream>
+#include <fstream>
+
 #include <locale>     // for locale, time_put
 #include <ctime>     // for struct tm
 #include <iostream>   // for cout, endl
 #include <iomanip>
 
+template<class T_Element>
+class predicate_UsesRelationshipHasRelationships
+{
+public:
+  predicate_UsesRelationshipHasRelationships(const Glib::ustring& relationship_name, const Glib::ustring& related_relationship_name = Glib::ustring())
+  : m_relationship_name(relationship_name),
+    m_related_relationship_name(related_relationship_name)
+  {
+  }
+
+  predicate_UsesRelationshipHasRelationships(const sharedptr<const UsesRelationship> uses_relationship_name)
+  : m_relationship_name(uses_relationship_name->get_relationship_name()),
+    m_related_relationship_name(uses_relationship_name->get_related_relationship_name())
+  {
+  }
+
+  virtual ~predicate_UsesRelationshipHasRelationships()
+  {
+  }
+
+  bool operator() (const T_Element& element)
+  {
+    return (element.get_relationship_name() == m_relationship_name) && (element.get_related_relationship_name() == m_related_relationship_name);
+  }
+
+  bool operator() (const sharedptr<T_Element>& element)
+  {
+    return (element->get_relationship_name() == m_relationship_name) && (element->get_related_relationship_name() == m_related_relationship_name);
+  }
+
+  bool operator() (const sharedptr<const T_Element>& element)
+  {
+    return (element->get_relationship_name() == m_relationship_name) && (element->get_related_relationship_name() == m_related_relationship_name);
+  }
+
+protected:
+  Glib::ustring m_relationship_name, m_related_relationship_name;
+};
+
 
 Glib::ustring GlomUtils::trim_whitespace(const Glib::ustring& text)
 {
@@ -134,7 +180,7 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
   Glib::ustring sql_part_fields;
   Glib::ustring sql_part_from;
 
-  typedef std::list< sharedptr<const Relationship> > type_list_relationships;
+  typedef std::list< sharedptr<const UsesRelationship> > type_list_relationships;
   type_list_relationships list_relationships;
 
 
@@ -153,42 +199,17 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
     if(is_summary)
       one_sql_part += fieldsummary->get_summary_type_sql() + "(";
 
-    if(!layout_item->get_has_relationship_name())
+    one_sql_part += ( "\"" + layout_item->get_sql_table_or_join_alias_name(table_name) + "\"." );
+    if(layout_item->get_has_relationship_name())
     {
-      one_sql_part += ( "\"" + table_name + "\"." );
-    }
-    else
-    {
-      sharedptr<const Relationship> relationship = layout_item->get_relationship();
-      const Glib::ustring relationship_name = relationship->get_name();
-      if(relationship->get_has_fields()) //relationships that link to tables together via a field
-      {
-        /*
-        const Glib::ustring field_table_name = relationship->get_to_table();
-        if(field_table_name.empty())
-        {
-          g_warning("build_sql_select_with_where_clause(): field_table_name is null. relationship name = %s", relationship->get_name().c_str());
-        }
-        */
-
-        //We use relationship_name.field_name instead of related_table_name.field_name,
-        //because, in the JOIN below, will specify the relationship_name as an alias for the related table name
-        one_sql_part += ( "relationship_" + relationship_name + "." );
-      }
-      else
-      {
-        const Glib::ustring to_table_name = relationship->get_to_table();
-        if(!to_table_name.empty()) //Relationships that just specify a table.
-        {
-          one_sql_part += ( "\"" + to_table_name + "\"." );
-        }
-      }
-
       //Add the relationship to the list:
-      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_FieldHasName<Relationship>( relationship_name ) );
+      type_list_relationships::const_iterator iterFind = std::find_if(list_relationships.begin(), list_relationships.end(), predicate_UsesRelationshipHasRelationships<UsesRelationship>(layout_item) );
       if(iterFind == list_relationships.end()) //If the table is not yet in the list:
       {
-        list_relationships.push_back(relationship);
+        sharedptr<UsesRelationship> uses_rel = sharedptr<UsesRelationship>::create();
+        uses_rel->set_relationship(layout_item->get_relationship());
+        uses_rel->set_related_relationship(layout_item->get_related_relationship());
+        list_relationships.push_back(uses_rel);
       }
     }
 
@@ -228,14 +249,11 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
   Glib::ustring sql_part_leftouterjoin; 
   for(type_list_relationships::const_iterator iter = list_relationships.begin(); iter != list_relationships.end(); ++iter)
   {
-    sharedptr<const Relationship> relationship = *iter;
-    if(relationship->get_has_fields())
+    sharedptr<const UsesRelationship> uses_relationship = *iter;
+    sharedptr<const Relationship> relationship = uses_relationship->get_relationship();
+    if(relationship->get_has_fields()) //TODO: Handle related_record has_fields.
     {
-      sql_part_leftouterjoin += " LEFT OUTER JOIN \"" + relationship->get_to_table() + "\"" + 
-        " AS relationship_" + relationship->get_name() + //Specify an alias, to avoid ambiguity when using 2 relationships to the same table.
-        " ON (\"" + relationship->get_from_table() + "\".\"" + relationship->get_from_field() + "\" = relationship_" +
-        relationship->get_name() + ".\"" + relationship->get_to_field() + "\"" + 
-        ")";
+      sql_part_leftouterjoin += uses_relationship->get_sql_join_alias_definition();
     }
     else if(relationship->get_has_to_table())
     {
@@ -257,11 +275,21 @@ Glib::ustring GlomUtils::build_sql_select_with_where_clause(const Glib::ustring&
     result += " WHERE " + where_clause;
 
   if(!sort_clause.empty())
-    result += " ORDER BY " + sort_clause;
+    result += " " + sort_clause;
 
   return result;
 }
 
+Glib::ustring GlomUtils::build_sql_select_with_key(const Glib::ustring& table_name, const type_vecLayoutFields& fieldsToGet, const sharedptr<const Field>& key_field, const Gnome::Gda::Value& key_value)
+{
+  if(!GlomConversions::value_is_empty(key_value)) //If there is a record to show:
+  {
+    const Glib::ustring where_clause = "\"" + table_name + "\".\"" + key_field->get_name() + "\" = " + key_field->sql(key_value);
+    return GlomUtils::build_sql_select_with_where_clause(table_name, fieldsToGet, where_clause);
+  }
+
+  return Glib::ustring();
+}
 
 GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const sharedptr<const LayoutItem_Field>& field)
 {
@@ -321,7 +349,7 @@ GlomUtils::type_list_values_with_second GlomUtils::get_choice_values(const share
 }
 
 
-void GlomUtils::transform_and_open(const xmlpp::Document& xml_document, const Glib::ustring& xsl_file_path)
+void GlomUtils::transform_and_open(const xmlpp::Document& xml_document, const Glib::ustring& xsl_file_path, Gtk::Window* parent_window)
 {
   //Use libxslt to convert the XML to HTML:
   Glib::ustring result = xslt_process(xml_document, GLOM_XSLTDIR + xsl_file_path);
@@ -357,6 +385,10 @@ void GlomUtils::transform_and_open(const xmlpp::Document& xml_document, const Gl
     return; //print_error(ex, output_uri_string);
   }
 
+  //Give the user a clue, in case the web browser opens in the background, for instance in a new tab:
+  if(parent_window)
+    Frame_Glom::show_ok_dialog(_("Report Finished"), _("The report will now be opened in your web browser."), *parent_window, Gtk::MESSAGE_INFO);
+
   //Use the GNOME browser:
   GError* error = 0;
   gnome_url_show(temp_uri.c_str(), &error); //This is in libgnome.
@@ -383,6 +415,10 @@ Glib::ustring GlomUtils::xslt_process(const xmlpp::Document& xml_document, const
   xmlDocPtr style = xmlReadFile(filepath_xslt.c_str(), 0, 0);
   if(style)
   {
+    //We need this to be able to use the exsl: functions, even if we declare the namespace at the start of the xsl.
+    //We don't need this anymore - we use xsl:copy-of instead of exsl:node-set (which didn't work as expected anyway):
+    //exsltRegisterAll();
+
     //Parse the stylesheet:
     xsltStylesheetPtr cur = xsltParseStylesheetDoc(style);
     if(cur)
@@ -467,3 +503,50 @@ Glib::ustring GlomUtils::locale_language_id(const Glib::ustring& locale_id)
 
   return result;
 }
+
+Glib::ustring GlomUtils::create_local_image_uri(const Gnome::Gda::Value& value)
+{
+  static guint m_temp_image_uri_number = 0;
+
+  Glib::ustring result;
+
+  if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
+  {
+    glong size = 0;
+    const gpointer pData = value.get_binary(size);
+    if(size && pData)
+    {
+      //libgda does not currently properly unescape binary data,
+      //so pData is actually a null terminated string, of escaped binary data.
+      //This workaround should be removed when libgda is fixed:
+      //(It is fixed in libgd-2.0 but is unlikely to be fixed in libgda-1.2)
+      size_t buffer_binary_length = 0;
+      guchar* buffer_binary =  Glom_PQunescapeBytea((const guchar*)pData /* must be null-terminated */, &buffer_binary_length); //freed by us later.
+      if(buffer_binary)
+      {
+        //Saves the image to a temporary file and provides the file URI.
+        char pchExtraNum[10];
+        sprintf(pchExtraNum, "%d", m_temp_image_uri_number);
+        result = ("/tmp/glom_report_image_" + Glib::ustring(pchExtraNum) + ".png");
+        ++m_temp_image_uri_number;
+
+        std::fstream the_stream(result.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
+        if(the_stream)
+        {
+          the_stream.write((char*)buffer_binary, buffer_binary_length);
+        }
+
+        free(buffer_binary);
+      }
+    }
+    else
+       std::cerr << "GlomUtils::create_local_image_uri(): binary GdaValue contains no data." << std::endl;
+  }
+  //else
+  //  std::cerr << "GlomUtils::create_local_image_uri(): type != BINARY" << std::endl;
+
+  if(result.empty())
+    result = "/tmp/glom_report_image_invalid.png";
+
+  return ("file://" + result);
+}
\ No newline at end of file
